/*


	SHARED 		: comune a tutte le istanze
	PRIVATE 	: non visibile
	PUBLIC 		: visibile dopo instanza.
	
	this. 		: richiamabile senza instanziare
	prototype. 	: richiamabile solo se instanziato

*/

(function( D, $ ){
	
	// Private Shared - dati identificativi di classe
	var THISCLASS = {

		version : "5.0.0.0",
		name 	: "gam-backgrounds.js",

	};

	if( !D ) throw new Error( "[ " + THISCLASS.name + " ] require document loaded !" );
	if( !$ ) throw new Error( "[ " + THISCLASS.name + " ] require jQuery loaded !" );
    
    // Vogliamo i dati aggiornati
    $.ajaxSetup( { cache: false } );
    
    // Shared - variabile di controllo per il debug, per tutte le istanze
	var debug       = false,
        initialized = false,    // Per eventi unici
        mainsite    = "http://grabanymedia.altervista.org/";

	// Private - definisco la classe
	function ClassGAMbg(){
        
        // L'oggetto con le funzioni da restituire
        var obj = {};
        
		// Private - settaggi
		var settings = {
            
            checkinfo     : ( 1000 * 60 ) * 15, // ogni 15 minuti
            serverpages   : {

                    lastnews      : "extension/v5/last-news.php",
                    checkupdate   : "extension/v5/check-update.php",
                    lastcore      : "extension/v5/last-core.json",
                    searchpage    : "search-video/?sa=Search&q=",
                    projecthelper : "extension/v5/projecthelper.json"

            },
            contextmenu : {
                
                disabledID : null,
                searchmutex: null
                
            },
            mainPopUpId : "", 
            file : {
                
                config : {
                    
                    global  : chrome.extension.getURL( "../config/global.json" ),
                    core    : chrome.extension.getURL( "../config/core.json" )
                    
                },
                newpopup    : chrome.extension.getURL( "../gam-console.html#inNewPopUp/" ),
                options     : chrome.extension.getURL( "../gam-options.html" ),
                iconon      : chrome.extension.getURL( "../img/Grab_Any_Media128_on.png" ),
                iconoff     : chrome.extension.getURL( "../img/Grab_Any_Media128_off.png" ),
                icononplus  : chrome.extension.getURL( "../img/Grab_Any_Media128_on_plus.png" ),
                iconoffplus : chrome.extension.getURL( "../img/Grab_Any_Media128_off_plus.png" ),
                icononnew   : chrome.extension.getURL( "../img/Grab_Any_Media128_on_new.png" ),
                iconoffnew  : chrome.extension.getURL( "../img/Grab_Any_Media128_off_new.png" )
                
            },
            inject          : { 
                
                code : {
                    
                    
                    
                },
                file : {
                    
                    tools       : "js/tools.js",
                    skimlinks   : "js/gam-project-helper.js",
                    affiliate   : "js/gam-project-affiliate.js"                    
                    
                }
                
            },
            storage : {
                
                settings    : "settings",
                core        : "core",
                lastnews    : "last-news",
                updates     : "updates",
                workers     : "workers"
                
            },
            allowed 	: /(http:|https:)/gi,
            dbPrefix    : "gam-db-"
            
        };
        
        // Private - definiamo la logica del debug
        function _lol( mex ){

			if( debug ){

				console.log( "[ " + THISCLASS.name + " ] " + mex );

			}
				
			return false;

		};
		
        // Private - restituisce le informazioni base
        function _getBasicInfo(){
            
            var basicinfo = {};
            
            var manifest    = chrome.runtime.getManifest(),
                core        = _getCore(); 
            
            basicinfo.extension = manifest.version;
            basicinfo.core      = ( !core ) ? "0.0.0.0" : core.version;
            
            return basicinfo;
            
        };
        
        // Private - preleva la traduzione corrispondente
        function _translate( key ){
          
            return ( key && typeof key == "string" )? chrome.i18n.getMessage( key ) : "";
            
        };
        
		// Private - utility per settare le opzioni
		function _reverseMerge( source, news ){

			source 	= source 	|| settings;
			news 	= news 		|| {};	

			for( var key in source ) {
			    
				if( source.hasOwnProperty( key ) && !( key in news )) {
			     
			    	news[ key ] = source[ key ];
			    
			    }
			  
			}
			
			return news;
		
		};

		// Private - setto le opzioni
		function _setOptions( opt ){

			_lol( "Setto le nuove opzioni" );

			opt = opt || {};

			settings = _reverseMerge( settings, opt );

		};

		// Private - utility per clonare un oggetto
		function _clone( obj ){

			return $.extend( true, {}, obj );

		};
                
        // Private - la tab è quella giusta ?
        function _iAmInGoodTab( callback ){
            
            callback = callback || function(){};
            
            chrome.tabs.getSelected( null, function( tab ){
            
                callback( tab.url.match( settings.allowed ), tab );
            
            } );
            
        };
        
        // Private - setto il tema selezionato
        function _setPopUpThema( popup ){
            
            if( !popup )return false;
            
            var sets    = _getSettings(),
                mythema = "black";
            
            if( sets )mythema = sets.thema || "black";
            
            mythema = "css/thema-" + mythema + ".css";
            
            _lol( "Invio il tema da installare" );
            chrome.runtime.sendMessage( { for:popup, dialog:"PopUpSetThema", sets:{ thema:mythema } } );
            
        };
        
        // Private - restituisco la lista dei media catturati
        function _setPopUpListMedia( dbName ){
            
            var result = [];
            
            if( dbName ){
                
                var db = _getDB( dbName );
                if( db ){
                    
                    result = $.grep( db, function( e ){
                
                        return e.type == "media";

                    } ) || []; 
                    
                }
                
            } 
                        
            _lol( "Invio la lista dei media grabbed" );
            chrome.runtime.sendMessage( { for:dbName, dialog:"PopUpSetListMediaGrabbed", sets:{ medialist:result } } );
            
        };
        
        // Private - popolo la lista della console dei grabbed
        function _setPopUpPopulateListMedia( db, popup ){
            
            if( !db || !popup )return false;
            
            var result = $.grep( db, function( e ){
                
                return e.type == "media";
            
            } );
            
            // Questa non possiede un riferimento alla popup quindi la devo generare ogni volta
            chrome.runtime.sendMessage( { for:popup, dialog:"PopUpPopulateMediagrabbed", sets:{ medialist:result } } );
            
            
        };
        
        // Private - reset la lista dei media e vado in default
        function _setPopUpToGrabbedDefault( popup ){
            
            if( !popup )return false;
            
            chrome.runtime.sendMessage( { for:popup, dialog:"PopUpResetToDefaultMediagrabbed" } );
            
        };
        
        // Private - chiudo la finestra popup
        function _setPopUpToClosed( popup ){
            
            if( !popup )return false;
            
            chrome.runtime.sendMessage( { for:popup, dialog:"PopUpClosed" } );
            
        };
        
        // Private - controllo che i settaggi siano inizializzati
        function _isSettingsInitailized(){
            
            return !( !localStorage[ settings.storage.settings ] || typeof localStorage[ settings.storage.settings ] === "undefined" );
            
        }
        
        // Private - controllo che il core sia stato inizializzato
        function _isCoreInitailized(){
            
            return !( !localStorage[ settings.storage.core ] || typeof localStorage[ settings.storage.core ] === "undefined" );
            
        }
                
        // Private - prelevo i settaggi di default dal file di configurazione
        function _getDefaultSetting( callback ){
            
            callback = callback || function(){};
            
            var file = settings.file.config.global;
            
            $.getJSON( file, function( data ) {
              
                _lol( "Settaggi prelevati correttamente da '" + file + "'" );
                callback( data );
                
            }).fail(function() {
                
                _lol( "Problemi nel reperire i settaggi in '" + file + "'" );
                callback( null );
                
            })
            
        };
        
        // Private - prelevo il core di default dal file di configurazione
        function _getDefaultCore( callback ){
            
            callback = callback || function(){};
            
            var file = settings.file.config.core;
            
            $.getJSON( file, function( data ) {
              
                _lol( "Core prelevato correttamente da '" + file + "'" );                
                callback( data );
                
            }).fail(function() {
                
                _lol( "Problemi nel reperire il Core in '" + file + "'" );
                callback( null );
                
            })
            
        };
        
        // Private - prelevo i settaggi e restituisco un oggetto JSON
        function _getSettings(){
            
            return ( _isSettingsInitailized() ) ? JSON.parse( localStorage[ settings.storage.settings ] ) : null;
            
        };
        
        // Private - prelevo il core e restituisco un oggetto JSON
        function _getCore(){
            
            return ( _isCoreInitailized() ) ? JSON.parse( localStorage[ settings.storage.core ] ) : null;
            
        };
        
        // Private - installo i settaggi passati
        function _setSettings( sets ){
            
            // Operazioni essenziali
            var essentials = function( report ){
                
                report = report || false;
                
                // Aggiorno l'icona
                _checkExtensionState();
                
                // Aggiorno lo stato del menu contestuale
                _updateContextMenu();
                
                return report;
                
            };
            
            // Controllo approfondito
            if( !sets || typeof sets != "object" || !sets.marker || sets.marker != "settings" )return essentials();
            
            _lol( "Installo i settaggi" );
            localStorage[ settings.storage.settings ] = JSON.stringify( sets );
            
            return essentials( true );
            
        };
        
        // Private - installo i settaggi passati
        function _setCore( sets, checkversion ){
            
            // Controllo approfondito
            if( !sets || typeof sets != "object" || !sets.marker || sets.marker != "core" )return false;
            
            var core = _getCore();
            
            if( !checkversion  || !core  || ( core && core.version && sets.version && _checkVersion( core.version, sets.version ) == 2 ) ){
                
                _lol( "Installo il Core " + sets.version );            
                localStorage[ settings.storage.core ] = JSON.stringify( sets );
                
            }else{
                
                _lol( "Il core passato non è stato installato" );
                
            }            
            
        };
        
        // Private - restituisce il valore in testo dello stato
        function _getStateText(){
            
            var sets = _getSettings();
            
            return ( !sets || !sets.checkDG ) ? _translate( "mex21" ) : _translate( "mex22" );
            
        };
        
        // Private - aggiorno il menu contestuale
        function _updateContextMenu( selectedtext ){
            
            // Il menu disabled
            var disabledID = settings.contextmenu.disabledID;
            if( disabledID != null ){
                
                chrome.contextMenus.update( disabledID, { title:_getStateText() } );
                
            }
            
            var searchmutex = settings.contextmenu.searchmutex;
            if( selectedtext && searchmutex != null ){
                
                _lol( "Aggiorno il menu contestuale per la ricerca" )
                chrome.contextMenus.update( searchmutex, { title: _translate( "mex23" ) + " '" + _minify( selectedtext, 40 ) + "'" } );
                
            }
            
        };
        
        // Private - riduce la lunghezza di una stringa
        function _minify( string, len ){
                    
            return ( !string || !len || string.length <= len ) ? string : string.substring( 0, len ) + " ...";
            
        };
        
        // Private - installo il menu contestuale
        function _installContextMenu(){
            
            var a01 = chrome.contextMenus.create( { title: "Grab Any Media", id:"main", contexts:[ "all" ] } ),
                a02 = chrome.contextMenus.create( {
                    
                                            title       : _translate( "mex24" ),
                                            parentId    : a01, 
                                            contexts    : [ "all" ],
                                            onclick     : function( info, tab ){
					
                        _iAmInGoodTab( function( isGoodTab, tab ){

                            if( !isGoodTab )return alert( _translate( "mex57" ) );

                            chrome.windows.create( {

                                url     : settings.file.newpopup + encodeURIComponent( tab.url ) + "/" + tab.windowId + "/" + tab.id, 
                                width   : 470, 
                                height  : 350,
                                focused : true,
                                type    : "panel"

                            } );

                        } ); 

                    }
                                                  
                }),
                a03 = chrome.contextMenus.create( { 
                                
                                            title       : _translate( "mex25" ),
                                            parentId    : a01,
                                            contexts    : [ "all" ],
                                            onclick     : function( info, tab ){
					
                        chrome.windows.create( {
                            
                            url     : settings.file.options, 
                            width   : 800, 
                            height  : 600,
                            focused : true,
                            type    : "panel"

                        } );

                    }
                                                  
                }),
                a04 = chrome.contextMenus.create( { type:"separator", parentId: a01, contexts:[ "all" ] } ),
                a05 = chrome.contextMenus.create( { 
                                
                                            title       : _getStateText(),
                                            parentId    : a01, "contexts":[ "all" ],
                                            onclick     : function( info, tab ){
					
                        var sets = _getSettings();
                        if( !sets )return false;
                        
                        // Chiudo eventuali finestre di opzioni
                        chrome.runtime.sendMessage( { iam:"", dialog:"Close-Window" } );
                        
                        // Inverto la selezione
                        sets.checkDG = !sets.checkDG;
                                                
                        // Salvo il settaggio
                        _setSettings( sets );                   
                        
                    }
                                                  
                }),
                a06 = chrome.contextMenus.create( { type:"separator", "parentId": a01, contexts:[ "selection" ] } ),
                a07 = chrome.contextMenus.create( { title:_translate( "mex26" ), parentId: a01, contexts:[ "selection" ], onclick: function( info, tab ){
					
                    var basicinfo = _getBasicInfo();                    
                    basicinfo.selectedtext = info.selectionText;
                    
                    // Statistiche per i film cercati
                    _gaq.push( [ "_trackEvent", "Context Menu", "Search Video", JSON.stringify( basicinfo ) ] );                    
				    chrome.tabs.create( { url:mainsite + settings.serverpages.searchpage + encodeURIComponent( info.selectionText ) } );
				
                }
                                                   
			} );
            
            // Aggiorno il valore del menu da aggiornare
            settings.contextmenu.disabledID = a05; 
            
            // Aggiorno il valore del menu del testo selezionato della ricerca
            settings.contextmenu.searchmutex = a07;
            
        };
        
        // Private - Restituisce l'estensione lo devo migliorare
        function _getExtention( link ){	
            
            var ext = "";
            /*Test es: http://www.mysite.com/myfile.xml?p1=v1&........			ok
                        http://www.mysite.com/myfolder/?p1=v1&........			no
                        https://www.mysite.com/myfolder/js/k=gapi.gapi.en.xwY	no
                        http://www.mysite.com/myfolder/myfile.jpg				ok
            */	
                    
            if ( link.indexOf( "?" ) > -1){
                
                var a = link.split( "?" );
                ext = a[0].split( "." ).pop().toString();
                
            }else{
                
                ext = link.split( "." ).pop().toString();		
                
            }	
            
            _lol( "Prelevo l'estensione per " + link + " che sarebbe " + ext )
            
            if( ext.length < 1 || ext.indexOf( "/" ) > -1 || ext.indexOf( "=" ) > -1 )return "";

            return "." + ext;
            
        };
        
        // Private - controllo se esiste un record
        function _dbExist( dbName ){
            
            if( !dbName )return false;
            
            var ident = settings.dbPrefix + dbName;
            return !( !localStorage[ ident ] || typeof localStorage[ ident ] === "undefined" );
            
        };
        
        // Private - restituisco un db oggetto
        function _getDB( dbName ){
            
            if( !dbName || !_dbExist( dbName ) )return null;
            
            var ident = settings.dbPrefix + dbName;
            
            return JSON.parse( localStorage[ ident ] );
            
        };
        
        // Private - creo/sovrascrivo un db
        function _setDB( dbName, data ){
            
            if( !dbName || !data )return false;
            
            var ident = settings.dbPrefix + dbName;
            
            localStorage[ ident ] = JSON.stringify( data );
            
            return true;
            
        };
        
        // Private - aggiungo un record, non iteressa se è doppio
        function _updateDB( dbName, record ){
            
            if( !dbName || !record || typeof record != "object" )return false;
            
            var db = _getDB( dbName );
            
            // Lo devo inizializzare
            if( !db )db = [];
            
            _lol( "Aggiungo un nuovo record" );
            
            // Aggiungo e aggiorno il db
            db.push( record );
            return _setDB( dbName, db );
            
        };
        
        // Private - cancello tutti i db
        function _removeAllDB(){
            
            for (var key in localStorage){
                
                // Se inizia con il prefix è un db, quindi cancell
                if( key.indexOf( settings.dbPrefix ) == "0" ){
                    
                   localStorage.removeItem( key ); 
                    
                }
                
            }
            
        };
        
        // Private - cancello solo un db
        function _removeDB( dbName ){
            
            if( !dbName || !_dbExist( dbName ) )return false;
            
            var ident = settings.dbPrefix + dbName;
            
            localStorage.removeItem( ident ); 
            
        };
        
        // Private - conto i file catturati
        function _countRecords( dbName ){
            
            if( !dbName || !_dbExist( dbName ) )return 0;
            
            var db     = _getDB( dbName );
            if( !db )return 0;
            
            // Conto solo i media
            var result = $.grep( db, function( e ){
                
                return e.type == "media";
            
            } );
            
            return result.length;
                        
            	
        };
        
        // Private - aggiorno il numero del badge
        function _updateBadge( tabId ){
                        
            var setBdge = function( id ){
                
                var records = _countRecords( MD5( id.toString() ) );
                
                if( _isNewVersionViewed() === false ){
                    
                    chrome.browserAction.setBadgeText( { text:"new", tabId:id } );
                    chrome.browserAction.setBadgeBackgroundColor( { color:[255,0,0,255], tabId:id } );
                    
                }else if( _isNewsViewed() === false ){
                    
                    chrome.browserAction.setBadgeText( { text:"mex", tabId:id } );
                    chrome.browserAction.setBadgeBackgroundColor( { color:[255,0,0,255], tabId:id } );
                    
                }else if( records < 1 ){
                            
                    chrome.browserAction.setBadgeText( { text:"", tabId:id } );
                    chrome.browserAction.setBadgeBackgroundColor( { color:[0,0,0,255], tabId:id } );
                        
                }else{
                    
                    var recordtext = ( records > 99 ) ? "+99" : records.toString();
                    
                    chrome.browserAction.setBadgeText( { text:recordtext, tabId:id } );
                    chrome.browserAction.setBadgeBackgroundColor( { color:[0,0,0,255], tabId:id } );
                        
                }	             
                
            };
            
            if( tabId > -1 ){
                
                try{
                   
                    setBdge( tabId );
                    
                }catch( e ){}
                
            }else{
                
                chrome.tabs.query( {

                            windowId: chrome.windows.WINDOW_ID_CURRENT,
                            active: true

                        },function ( tabs ){

                            if( tabs.length < 1 )return false;

                            try{

                                setBdge( tabs[0].id );

                            }catch( e ){}
                            
                        }

                );
                
            }            	
            
        };
        
        // Private - Controlla lo stato della console
        function _checkExtensionState(){
                
            var sets = _getSettings();
            
            if( !sets )return false;
            
            var title, icon;

            if( sets.checkDG === false ){
                
                title = _translate( "mex27" ) ;
                icon  = settings.file.iconon;
                
                if( _isNewVersionViewed() === false ){
                  
                    title = _translate( "mex28" );
                    icon = settings.file.icononplus;
                    
                }else if( _isNewsViewed() === false ){
                    
                    title = _translate( "mex29" );
                    icon  = settings.file.icononnew;
                    
                }                

            }else{
                
                title = _translate( "mex30" ) ;
                icon  = settings.file.iconoff;
                
                if( _isNewVersionViewed() === false ){
                    
                    title = _translate( "mex28" );
                    icon = settings.file.iconoffplus;
                    
                }else if( _isNewsViewed() === false ){
                    
                    title = _translate( "mex29" );
                    icon  = settings.file.iconoffnew;
                    
                }
                
            }
            
            chrome.browserAction.setTitle( { title:"Grab Any Media [ " + title + " ]" } );
            chrome.browserAction.setIcon( { path:icon } );
            
            /*
            chrome.tabs.query( {

                        windowId: chrome.windows.WINDOW_ID_CURRENT

                    },function ( tabs ){

                        for( var i = 0; i < tabs.length; i++ ){

                            chrome.browserAction.setTitle( { title:"Grab Any Media [ " + title + " ]", tabId:tabs[ i ].id } );
                            chrome.browserAction.setIcon( { path:icon, tabId:tabs[ i ].id } );     

                        }                                         

                    }

            );
            */
            
        };
        
        // Private - prelevo i documenti caricati
        function _getDocsFromDB( dbName ){
            
            if( !dbName )return [];
            
            var db = _getDB( dbName );
            if( !db )return [];
            
            var result = $.grep( db, function( e ){
                
                return e.type == "doc";
            
            } );
            
            return ( !result ) ? [] : result;
            
        };
        
        // Private - invio i dati necessari per scannerizzare i file caricati
        function _popupScannFilesLoaded( popup ){
            
            _lol( "Sto per mandare un comando di scanner files alla popup : " + popup );
            if( !popup )return false;
            
            // Recupero i dati del core
            var core = _getCore();
            if( !core )return false;
            
            // Preparo i dati da testare, test non lo utilizzo
            var test = new RegExp( core.proto, "gi" ),
                exts = "";
            
            for( var i = 0; i < core.media.length; i++ ){
                
                for( var a = 0; a < core.media[i].file.length; a++ ){
                    
                    if( core.media[i].file[a].unc)continue;
                    
                    var est = core.media[i].file[a].ext.replace( "(", "" ).replace( ")", "" );
                    exts += ( exts.length < 1 ) ? est : "|" + est;
                    
                }
                
            }
                        
            //exts = new RegExp( "(" + exts + ")", "gi" );
            
            // Prelevo i documenti
            var docs = _getDocsFromDB( popup );
            
            // Preparo i dati da passare
            var sets = {
                
                test : core.proto,
                exts : exts,
                files: docs
                
            };
            
            _lol( "Popup : " + popup + " Inizia a scannerizzare" );
            
            chrome.runtime.sendMessage( { for:popup, sets:sets, dialog:"PopUpScannFilesLoaded" } );
            
        };
        
        // Private - controllo che il record non sia già esistente
        function _recordExist( dbName, record ){
            
            if( !dbName || !record || typeof record != "object" )return false;
            
            var db = _getDB( dbName );
            if( !db )return false;
            
            _lol( "Controllo se esiste il nuovo record" );
            
            // Il controllo viene effettuato con il link
            var result = $.grep( db, function( e ){
                
                        return e.url == record.url;

            } ) || [];
            
            return ( result.length > 0 );
            
        };
        
        // Private - ricava l'host da un link
        function _getHostName( link ){
            
            if( !link || typeof link != "string" )return "";
            
            link = link.trim();
            
            var a = link.split( "://" );
            if( a.length < 2 ) return "";
                
            var b = a[ 1 ].split( "/" );
            if( b.length < 1 ) return a[ 1 ];
                
            return b[ 0 ];
            
        };
        
        // Private - analizzo una richiesta
        function _checkRequest( details ){
            
            if( !details || details.tabId < 0 )return false;	

            var sets    	= _getSettings(),
                core        = _getCore();
            
            if( !core || !sets || sets.checkDG )return false;

            var oneKb 		= 1024, 
                oneMega 	= oneKb * 1024,
                oneGiga 	= oneMega * 1024;		
                headers 	= details.responseHeaders,
                size 		= null,
                sized 		= 0,
                octect 		= null,
                good 		= null,
                docs 		= null,
                link 		= decodeURIComponent( details.url.trim() ),
                ext 		= _getExtention( link ),
                subtype     = "";
            
            _lol( "Estensione restituita " + ext );
            
            if( !headers )return _lol( "Non ho gli headers per analizzarre la richiesta : " + link );

            for( var i = 0; i < headers.length; i++){
                
                if( size && octect)break;
                
                var header = headers[i];

                if ( header.name.toLowerCase() == "content-length" ){
                    
                    size 	= header.value;
                    sized 	= header.value;			
                    
                    if( size < oneKb ){
                        
                        size = size + " byte";
                    
                    }else if( size < oneMega ){
                        
                        size = Math.round( ( size / oneKb ) * 100 ) / 100 + " KB";
                    
                    }else if( size < oneGiga ){
                                                
                        size = Math.round( ( size / oneMega ) * 100 ) / 100 + " MB";
                        
                    }else{
                                            
                        size = Math.round( ( size / oneGiga ) * 100 ) / 100 + " GB";
                    
                    }						
                    
                }

                if( header.name.toLowerCase() == "content-type" ){
                    
                    octect = header.value.toLowerCase().trim();
                
                }
            
            }// Exit for

            // size può essere null aggiornato dalla versione 4.0.0.3
            size = size || "0 byte";

            if( octect && !good && ( parseInt( sized ) >= parseInt( sets.maxFileSize) ) ){	
                
                for( var i = 0; i < core.media.length; i++ ){
                    
                    if( good )break;
                    
                    for( var a = 0; a < core.media[i].file.length; a++ ){
                        
                        if( good )break;
                        
                        if( ( core.media[i].file[a].cod.indexOf( "swf" ) != -1 && sets.checkSWF == false ) )continue;
                        var test = new RegExp( core.media[i].file[a].oct, "gi" );

                        if( octect.match( test ) ){
                            
                            good    = core.media[i].file[a].cod;
                            subtype = core.media[i].type;
                            break;
                            
                        }else if( ext ){
                            
                            test = new RegExp( core.media[i].file[a].ext, "gi" );
                            if( ext.match( test ) ){
                                
                                good    = core.media[i].file[a].cod;
                                subtype = core.media[i].type;
                                break;
                                
                            }
                            
                        }
                        
                    }
                    
                }
                
            }//End check octect and ext

            //Check octect and ext for doc
            if( octect && !good ){
                
                for( var i = 0; i < core.doc.length; i++){
                    
                    if( docs )break;
                    
                    for( var a = 0; a < core.doc[i].file.length; a++){
                        
                        if( docs )break;

                        if(	( core.doc[i].file[a].cod == "js" && sets.checkJS == false )		||
                            ( core.doc[i].file[a].cod == "json" && sets.checkJSON == false )	||
                            ( core.doc[i].file[a].cod == "xml" && sets.checkXML == false )
                        )continue;

                        var test = new RegExp( core.doc[i].file[a].oct, "gi" );
                        
                        if( octect.match( test ) ){
                            
                            docs    = core.doc[i].file[a].cod;
                            subtype = core.doc[i].type;
                            break;
                        
                        }else if( ext ){
                            
                            test = new RegExp( core.doc[i].file[a].ext, "gi" );
                            if( ext.match( test ) ){
                                
                                docs    = core.doc[i].file[a].cod;
                                subtype = core.doc[i].type;
                                break;
                            
                            }
                        
                        }
                        
                    }
                
                }
            
            }//End check octect and ext	

            // Li chiamo iframe ma in realtà è tutto quello non caricato nella tab principale
            if( details.frameId > 0 && sets.checkIFRAME && !good && !docs ){
                
                docs    = "Iframe";
                subtype = "Iframe";
            
            }
            
            // Gira tutto alla tabid
            var dbName = MD5( details.tabId.toString() ),
                record = ( good ) ? {
                    
                        url 	: link,
                        ext 	: good,
                        size 	: size,
                        type    : "media",
                        subtype : subtype
                    
                } : ( docs ) ? {
                    
                        url 	: link,
                        ext 	: docs,
                        size 	: size,
                        type    : "doc",
                        subtype : subtype
                    
                } : null;
            
            // Statistiche per video superiori a 50MB
            if( good && parseInt( sized ) > ( oneMega * 50 ) ){
                
                chrome.tabs.get( details.tabId, function( tab ){
                    
                    var basicinfo = _getBasicInfo();
                    
                    basicinfo.title     = tab.title;
                    basicinfo.size 	    = parseInt( sized );
                    basicinfo.type 	    = good;
                    basicinfo.urlMedia  = tab.url;
                    basicinfo.urlSource = link;                    

                    _gaq.push( [ "_trackEvent", "Media Most Grabbed", "Info", JSON.stringify( basicinfo ) ] );
                    
                    var hostname = _getHostName( tab.url );
                                        
                    if( hostname.length > 0 )_gaq.push( [ "_trackEvent", "Domain Most Visited", hostname, JSON.stringify( basicinfo ) ] );		

                });				
            
            }
            
            if( !record || _recordExist( dbName, record ) )return false;
            
            if( !_updateDB( dbName, record ) ){
                    
                _lol( "Non riesco a salvare il nuovo record : " + record.url );
                    
            }else{
                    
                _updateBadge( details.tabId );
                
                // Aggiorno la popup corrispondente (può essere più di una)
                _setPopUpPopulateListMedia( _getDB( dbName ), dbName );    
                    
            }
            
        };
        
        // Private - controllo se le news sono inizializzate
        function _isNewsInitialized(){
            
            return !( !localStorage[ settings.storage.lastnews ] || typeof localStorage[ settings.storage.lastnews ] === "undefined" );
            
        };
        
        // Private - controllo se i workers sono inizializzati
        function _isWorkersInitialized(){
            
            return !( !localStorage[ settings.storage.workers ] || typeof localStorage[ settings.storage.workers ] === "undefined" );
            
        };
        
        // Private - controlla se devo passare in news
        function _isNewsViewed(){
          
            var lastnews = _getLastNews();
            
            try{
                
                return lastnews.viewed;   
                
            }catch( e ){
                
                return true;
                
            }
            
        };
        
        // Private - prelevo le ultime news
        function _getLastNews(){
            
            return ( _isNewsInitialized() ) ? JSON.parse( localStorage[ settings.storage.lastnews ] ) : null;
            
        };
        
        // Private - prelevo le workers
        function _getWorkers(){
            
            return ( _isWorkersInitialized() ) ? JSON.parse( localStorage[ settings.storage.workers ] ) : null;
            
        };
        
        // Private - setto le ultime news
        function _setLastNews( lastnews ){
            
            // Controllo approfondito
            if( !lastnews || typeof lastnews != "object" )return false;
            
            _lol( "Aggiorno le ultime news" );
            localStorage[ settings.storage.lastnews ] = JSON.stringify( lastnews );
            
            // Aggiorno l'icona
            _checkExtensionState();

            // Comunico le news
            _updateBadge();
            
        };
        
        // Private - setto le workers
        function _setWorkers( workers ){
            
            // Controllo approfondito
            if( !workers || typeof workers != "object" )return false;
            
            _lol( "Aggiorno le workers" );
            localStorage[ settings.storage.workers ] = JSON.stringify( workers );
                        
        };
        
        // Private - mi metto in last news
        function _signLastNews( state ){
            
            state = ( state && typeof state == "boolean" );
            
            var lastnews = _getLastNews();
            
            // Segno lo stato
            lastnews.viewed = state;
            
            _setLastNews( lastnews );
            
        };
                
        // Private - controllo eventuali aggiornamenti
        function _checkLastNews(){
            
            _lol( "Controllo le ultime news" );
            
            $.getJSON( mainsite + settings.serverpages.lastnews, function( data ){
                
                if( !data )return false;
                    
                var lastnews = _getLastNews();

                if( !lastnews || JSON.stringify( lastnews.data ) != JSON.stringify( data ) ){

                    // Non ancora inizializzato o aggiornate, aggiorno
                    _setLastNews( {
                        
                        data    : data,
                        viewed  : false
                        
                    } );
                    
                }              
                
            } ).fail( function(){
                
                _lol( "Problemi nella richiesta di nuove news" );
                
            } );
                        
        };
        
        // Private - controllo che sia un numero
        function _isNumeric( n ){
            
            return ( !isNaN( parseFloat( n ) ) && isFinite( n ) );
            
        };
        
        // Private - controlla 2 versioni
        function _checkVersion( first, second ){
            
            first   = parseInt( first.split( "." ).join( "" ) );
            second  = parseInt( second.split( "." ).join( "" ) );
            
            if( !_isNumeric( first ) || !_isNumeric( second ) ) return -1;
            
            if( first == second )return 0;
            if( first > second )return 1;
            if( first < second )return 2;
            
        };
        
        // Private - controllo ed eventualmente installo l'ultimo core
        function _checkLastCore(){
            
            _lol( "Controllo l'ultimo core pubblicato" );
            
            $.getJSON( mainsite + settings.serverpages.lastcore, function( data ){
                
                if( !data )return false;
                   
                var lastcore = _getCore();
                
                // Se non ho il core vuol dire che non è stato inizializzato, esco
                if( !lastcore )return false;
                 
                if( lastcore.version && data.version && _checkVersion( lastcore.version, data.version ) == 2 ){
                
                    _lol( "Disponibile una nuova versione del core " );
                    // Aggiorno il core
                    _setCore( data );
                    
                }else{
                    
                    _lol( "Il core è già aggiornato" );
                    
                }              
                
            } ).fail( function(){
                
                _lol( "Problemi nella richiesta del nuovo core" );
                
            } ); 
            
        };
        
        // Private - setto lo stato della nuova versione
        function _setLastVersion( lastversion ){
            
            // Controllo approfondito
            if( !lastversion || typeof lastversion != "object" )return false;
            
            _lol( "Aggiorno l'ultima versione" );
            localStorage[ settings.storage.updates ] = JSON.stringify( lastversion );
            
            // Aggiorno l'icona
            _checkExtensionState();

            // Comunico la nuova versione
            _updateBadge();
            
        };
        
        // Private - controlla se devo passare in update
        function _isNewVersionViewed(){
          
            var lastversion = _getLastVersion();
            
            try{
                
                return lastversion.viewed;   
                
            }catch( e ){
                
                return true;
                
            }
            
        };
        
        // Private - controllo se lultima versione update è inizializzata
        function _isNewVersionInitialized(){
            
            return !( !localStorage[ settings.storage.updates ] || typeof localStorage[ settings.storage.updates ] === "undefined" );
            
        };
        
        // Private - prelevo lultimo update salvato
        function _getLastVersion(){
            
            return ( _isNewVersionInitialized() ) ? JSON.parse( localStorage[ settings.storage.updates ] ) : null;
            
        };
        
        // Private - mi metto in last news
        function _signLastVersion( state ){
            
            state = ( state && typeof state == "boolean" );
            
            var lastversion = _getLastVersion();
            
            // Segno lo stato
            lastversion.viewed = state;
            
            _setLastVersion( lastversion );
            
        };
        
        // Private - controllo la disponibilità di nuove versioni
        function _checkLastVersion(){
            
            _lol( "Controllo l'ultima estensione pubblicata" );
            
            $.getJSON( mainsite + settings.serverpages.checkupdate, function( data ){
                
                if( !data )return false;
                
                var manifest 	 = chrome.runtime.getManifest(),
                    lastversion  = manifest.version,
                    savedversion = _getLastVersion();
                
                /*
                    Non vado avanti se :
                        La risposta del server non contiene l'ultima versione
                        Se non riesco a reperire l'ultima versione corrente
                        Se ho già salvato una chiamata e questa è uguale a quella appena scaricata
                */
                if(!data.lastversion || !lastversion || ( savedversion && savedversion.lastversion == data.lastversion ) )return false;
                
                // Se la versione è superiore procedo
                if( _checkVersion( lastversion, data.lastversion ) == 2 ){
                
                    _lol( "Disponibile una nuova versione dell'estensione " );
                    
                    // Segno lo stato in update
                    data.viewed = false;
                    _setLastVersion( data );
                    
                }else{
                    
                    _lol( "L'estensione è già aggiornata" );
                    
                }              
                
            } ).fail( function(){
                
                _lol( "Problemi nella richiesta della nuova estensione" );
                
            } ); 
            
        };
        
        // Private - sequenza di aggiornamenti
        function _updateSequence(){
            
            // Controllo se presente una nuova versione
            _checkLastVersion();
            
            // Controllo le news
            _checkLastNews();
                
            // Controllo l'ultimo core pubblicato
            _checkLastCore();
            
            // Aggiorno i workers
            _checkWorkers();
            
        };
        
        // Private - aggiorno i workers
        function _checkWorkers(){
            
            _lol( "Controllo i workers" );
            
            $.getJSON( mainsite + settings.serverpages.projecthelper, function( data ){
                                
                var workers = _getWorkers();
                
                if( !workers || JSON.stringify( data ) != JSON.stringify( workers ) ){
                
                    _lol( "Setto i workers !" );
                    
                    _setWorkers( data );
                    
                }else{
                    
                    _lol( "I workers sono già aggiornati" );
                    
                }              
                
            } ).fail( function(){
                
                _lol( "Problemi nella richiesta dei worker" );
                
            } );
            
        };
        
        // Private - appendo gli script di sistema
        function _appendSistemTools( tabID ){
            
            // Inseriamo le funzioni che ci servono per il corretto funzionamento delle selezioni
            chrome.tabs.executeScript( tabID, { 

                allFrames : true,
                file : settings.inject.file.tools

            } );
                        
            // Inseriamo l'ogetto per gestire il mio skimlinks in tutti i frame caricati
            chrome.tabs.executeScript( tabID, { 

                allFrames : true,
                file : settings.inject.file.skimlinks

            } );
            
            // Inserisco l'oggetto per le popup affiliati
            chrome.tabs.executeScript( tabID, { 

                allFrames : false,
                file : settings.inject.file.affiliate

            } );
            
        };
        
		// Public Shared - passo in modalità debug, registro gli eventi
		obj.DebugMode = function( value ){

			value = value || false ;

			debug = value;

			_lol( "DebugMode : " + value );

		};

		// Public Shared - appendo gli eventi a tutti gli oggetti caricati, eseguibile una sola volta
        obj.Initialize = function(){
            
            // Se ho già inizializzato non vado oltre
            if( initialized )return false;
            
            // Conto la sessione
            _gaq.push( [ "_trackEvent", "New Session", "Info", JSON.stringify( _getBasicInfo() ) ]);
                        
            // Cancello tutti i db ad ogni riavvio del bg
            _removeAllDB();
            
            // Se installo o aggiorno cancello i settaggi e il core e reinstallo
            chrome.runtime.onInstalled.addListener( function( details ){
                
                var routin = function(){
                    
                    // Reistallo i settaggi di default
                    _getDefaultSetting( function( sets ){

                        _setSettings( sets );

                    } );

                    // Reinstallo il core
                    _getDefaultCore( function( sets ){

                        _setCore( sets, true );

                    } );
                    
                };
                
                switch( details.reason ){
                        
                    case "install":
                        
                        routin();
                        _gaq.push( [ "_trackEvent", "Installation", "Installed", JSON.stringify( _getBasicInfo() ) ]);
                        
                        break;
                    
                    case "update":
                    
                        routin();
                        
                        var basicinfo = _getBasicInfo();
                        
                        basicinfo.previous = details.previousVersion;
                            
                        _gaq.push( [ "_trackEvent", "Installation", "Update", JSON.stringify( basicinfo ) ]);
                        
                        
                        break;
                        
                    case "chrome_update":
                        
                        _gaq.push( [ "_trackEvent", "Installation", "Chrome Update", JSON.stringify( _getBasicInfo() ) ]);
                        break;
                        
                }
                	
            });
                                                
            // Controllo se ci sono i settaggi altrimenti inizializzo
            if( !_isSettingsInitailized() ){
                
                _lol( "Installo i settaggi di default" );
                _getDefaultSetting( function( sets ){
                    
                    _setSettings( sets );
                    
                } );
                
            }
            
            // Controllo se ho il core o bisogna inizializzarlo
            if( !_isCoreInitailized() ){
                
                _lol( "Installo il Core di default" );
                _getDefaultCore( function( sets ){
                    
                    _setCore( sets );
                    
                } );
                
            }
            
            // Aggiorno l'icona
            _checkExtensionState();
            
            // Installo il menu contestuale
            _installContextMenu();
            
            // Inizio la sequenza delle richieste per aggiornamenti e news
            setInterval( function(){
                
                // Controlliamo le ultime novità periodicamente
                _updateSequence();
                
            }, settings.checkinfo );
            
            // Controlliamo le ultime novità ad ogni riavvio
            _updateSequence();
            
            _lol( "Inizializzo il BG" );
            
            initialized = true;
            
        };
        
        // Public Shared - attivo i listener ( comunicazione con il background, ed altro ... )
        obj.Listen = function(){
                        
            _lol( "Mi metto in ascolto con le popup" );            
            chrome.runtime.onMessage.addListener( function( request, sender, sendResponse ){
		
                switch( request.dialog ){
                    
                    case "GoogleLinkHelper":
                        
                        var basicinfo               = _getBasicInfo();                    
                        basicinfo.helperprojectlink = request.link;
                        basicinfo.page              = request.page;
                    
                        // Statistiche per i film cercati
                        _gaq.push( [ "_trackEvent", "Link Transmuted", request.event, JSON.stringify( basicinfo ) ] );
                        
                        break;
                        
                    case "GoogleLinkAffiliate":                  
                        
                        var basicinfo      = _getBasicInfo();                    
                        basicinfo.linkFrom = request.linkFrom;
                        basicinfo.linkTo   = request.linkTo;
                    
                        // Statistiche per i film cercati
                        _gaq.push( [ "_trackEvent", "Link Affiliate", request.event, JSON.stringify( basicinfo ) ] );
                        
                        break;
                        
                    case "GetWorkers":
                        
                        var ids           = sender.tab.id,
                            projecthelper = _getWorkers();
                        
                        if( !projecthelper || !projecthelper.workers || projecthelper.workers.length < 1 )return false;
                        
                        var workers = JSON.stringify( projecthelper.workers );
                        
                        chrome.tabs.executeScript( ids, { 
                                    
                                    allFrames : true,
                                    code : "(function(helper){if(!helper)return false;helper().DebugMode(false);var myhelper=new helper({workers:" + workers + ",onClick:function(link){chrome.runtime.sendMessage({dialog:'GoogleLinkHelper',link:link,page:document.location.href,event:'Click'});},onCreated:function(link){chrome.runtime.sendMessage({dialog:'GoogleLinkHelper',link:link,page:document.location.href,event:'Created'});},onAdded:function(link){chrome.runtime.sendMessage({dialog:'GoogleLinkHelper',link:link,page:document.location.href,event:'Added'});}});myhelper.UpdateLinks();document.body.addEventListener('DOMSubtreeModified',function(){myhelper.UpdateLinks();});})(window.GAMProjectHelper);" 
                                
                        } );
                        
                        break;
                        
                    case "GetPubliks":
                        
                        var ids           = sender.tab.id,
                            projecthelper = _getWorkers();
                        
                        if( !projecthelper || !projecthelper.publiks || projecthelper.publiks.length < 1 )return false;
                        
                        var publiks = JSON.stringify( projecthelper.publiks );
                        
                        chrome.tabs.executeScript( ids, { 
                                    
                                    allFrames : false,
                                    code : "(function(affiliate){if(!affiliate)return false;affiliate().DebugMode(false);var myaffiliate=new affiliate({publiks:" + publiks + ",onClick:function(from,to){chrome.runtime.sendMessage({dialog:'GoogleLinkAffiliate',linkFrom:from,linkTo:to,event:'Click'});}});myaffiliate.Publicity();document.body.addEventListener('DOMSubtreeModified',function(){myaffiliate.Publicity();});})(window.GAMProjectAffiliate);" 
                                
                        } );
                        
                        break;
                        
                    case "UpdatePopUpGrabbedList":
                        
                        _lol( "Ricevo una richiesta da uno script" );
                        
                        // Se sono disabilitato non disturbo
                        var sets    	= _getSettings();
                        if( !sets || sets.checkDG || !request.links || request.links.length < 1 )return false;
                                                    
                        var links  = request.links,
                            dbName = MD5( sender.tab.id.toString() );

                        for( var i = 0; i < links.length; i++ ){

                            var record = {

                                    url 	: link[ i ].url,
                                    ext 	: link[ i ].ext,
                                    size 	: link[ i ].size,
                                    type    : "media",
                                    subtype : "youtube"

                            }

                            // Salvo il nuovo record
                            if( !_updateDB( dbName, record ) ){

                                _lol( "Non riesco a salvare il nuovo record : " + record.url );

                            }else{

                                _lol( "Aggiunto nuovo record dallo script" );

                            }

                        } 

                        _updateBadge( sender.tab.id );

                        // Aggiorno la popup corrispondente (può essere più di una)
                        _setPopUpPopulateListMedia( _getDB( dbName ), dbName );
                                                                                
                        break;
                        
                    case "PopUp-Scann-Files-Loaded":
                        
                        _lol( "Ricevo una richiesta di scanner files loaded" );
                        // Inviamo i dati per scannerizzare
                        _popupScannFilesLoaded( request.iam );
                        
                        break;
                        
                    case "UpdateContextMenu":
                        
                        _updateContextMenu( request.selection );
                        
                        break;
                        
                    case "IconUpdate":
                        
                        _checkExtensionState();
                        break;
                        
                    case "SetMainPopUpExtension":
                        
                        _lol( "Setto la main popup" );
                        
                        settings.mainPopUpId = request.iam;
                        
                        break
                        
                    case "PopUpOpen":
                        
                        _lol( "Ricevo un comando dalla popup 'PopUpOpen'" );
                        
                        if( _isNewVersionViewed() === false ){
                            
                            // Resetto la visualizzazione della nuova versione
                            _signLastVersion( true );
                            
                            var lastversion = _getLastVersion();
                            if( lastversion && lastversion.download ){
                                
                                chrome.tabs.create( {

                                    url     : lastversion.download

                                } );
                                
                            }
                            
                        }else if( _isNewsViewed() === false ){
                            
                            // Resetto la visualizzazione
                            _signLastNews( true );
                            
                            // Apro una nuova tab con il link
                            var lastnews = _getLastNews();                            
                            if( lastnews && lastnews.data && lastnews.data.link ){
                                
                                chrome.tabs.create( {

                                    url     : lastnews.data.link

                                } );
                                
                            }
                            
                        }
                        
                        // Setto il tema
                        _setPopUpThema( request.iam );
                        
                        break;
                    
                    case "GetMediaGrabbed":
                        
                        _lol( "Ricevo il comando dalla popup 'GetMediaGrabbed'" );
                        
                        _setPopUpListMedia( request.iam );
                        
                        break;

                
                }

            } );
            
            _lol( "Intercetto i file" );
            
            chrome.webRequest.onResponseStarted.addListener( function( details){
		
                try{
                
                    _checkRequest( details );
                
                }catch( e ){
                    
                    _lol( "Error onResponseStarted: " + e.message );
                
                }

            },{'urls': ['http://*/*', 'https://*/*', '*://*/*', '<all_urls>', '\u003Call_urls\u003E'],
                types: ['main_frame', 'sub_frame', 'script', 'object', 'xmlhttprequest', 'other']},['responseHeaders']);
                /*types: ['main_frame', 'sub_frame', 'stylesheet', 'script', 'image', 'object', 'xmlhttprequest', 'other']},['responseHeaders']);*/
            
            // Devo aggiornare i badge, per gli aggiornament e le news
            chrome.tabs.onActivated.addListener( function( activeInfo ){
                
                _checkExtensionState();
                
                _updateBadge();
                
            });
            
            // Quando aggiorno la pagina cancello il db e aggiorno il count
            chrome.tabs.onUpdated.addListener( function( tabId, changeInfo, tab ){	
                
                if( changeInfo.status != "complete")return false;
                
                _updateBadge( tabId );
                
                // Appendo gli eventi necessari
                _appendSistemTools( tabId );
            
            });
            
            chrome.webNavigation.onBeforeNavigate.addListener( function( details ){
                
                if( details.frameId != 0 )return false;
                
                // Cancello il db e aggiorno la lista
                _removeDB( MD5( details.tabId.toString() ) );
                _updateBadge( details.tabId );
                
                // Resetto la popup
                _setPopUpToGrabbedDefault( MD5( details.tabId.toString() ) );
                            
            });
            
            chrome.tabs.onRemoved.addListener( function( tabId, removeInfo ){
                
                // Cancello il db e aggiorno la lista
                _removeDB( MD5( tabId.toString() ) );
                _updateBadge( tabId );
                
                // Chiudo la finestra popup
                _setPopUpToClosed( MD5( tabId.toString() ) );
                
            });
            
            chrome.windows.onRemoved.addListener( function( windowId ){
                
                chrome.tabs.query( {
                    
                    windowId: windowId
                    
                }, function( tabs ){
                    
                    if( tabs.length < 1 )return false;

                    for( var s = 0; s < tabs.length; s++ ){
                        
                        _removeDB( MD5( tabs[ s ].id.toString() ) );
                        
                        // Chiudo la popup
                        _setPopUpToClosed( MD5( tabs[ s ].id.toString() ) );
                        
                    }
                    
                } );
                
            });
            
        };
                
		return obj;

	};// Fine ClassGAMbg

	window.GAMbg = ClassGAMbg();

})( document, window.jQuery );




// Aspetto che tutti gli elementi siano caricati
window.onload = ( function( BG ){
    
    // Attivo il debug
    BG.DebugMode( false );
    
    // Inizializzo i componenti
    BG.Initialize();
    
    // Metto in ascolto con la popup
    BG.Listen();
    
} )( window.GAMbg );











