/*


	SHARED 		: comune a tutte le istanze
	PRIVATE 	: non visibile
	PUBLIC 		: visibile dopo instanza.

	this. 		: richiamabile senza instanziare
	prototype. 	: richiamabile solo se instanziato

*/

(function( D, $ ){

	// Private Shared - dati identificativi di classe
	var THISCLASS = {

		version : "5.0.0.0",
		name 	: "gam-console.js",

	};

	if( !D ) throw new Error( "[ " + THISCLASS.name + " ] require document loaded !" );
	if( !$ ) throw new Error( "[ " + THISCLASS.name + " ] require jQuery loaded !" );

    // Shared - variabile di controllo per il debug, per tutte le istanze
	var debug       = false,
        initialized = false;    // Per eventi unici

	// Private - definisco la classe
	function ClassGAMconsole(){
        
        // L'oggetto con le funzioni da restituire
        var obj = {};
        
        // Lista delle scann trovate
        var finded = [];
        
		// Private - settaggi
		var settings = {
            
            storage : {
                
                settings    : "settings",
                core        : "core"
                
            },
            file : {
                
                options     : chrome.extension.getURL( "../gam-options.html" ),
                details     : chrome.extension.getURL( "../gam-details.html" )
            
            },
            inject          : { 
                    
                advanced    : "js/advanced-source.js"
                
            },
            iam             : MD5( "MainPopUp" ), // Mutevole
            tabId           : -1,
            newPopUpTracker : "#inNewPopUp/",
            newDTTracker    : "#DataForDetails/",
            grabbedProto    : "<div id=\"%ID%\" class=\"item\"> <i class=\"circular %I% icon to-download\"></i> <div class=\"content\"> <div class=\"header\">%E% <em>%S% </em><em class=\"feedback events\" style=\"display:none\"></em></div><i class=\"small icon copy\"></i><a href=\"%LL%\" class=\"link\" target=\"_blank\">%L%</a></div></div>",
            findedProto     : "<div class=\"item\"> <i class=\"circular browser icon\" data-index=\"%I%\"></i> <div class=\"content\"> <div class=\"header\">%H%</div><p class=\"snapshot\">%S%</p></div></div>"

		};
        
        // Public - Rende univoco un valore oggetto in un array
        Array.prototype.uniqueObj = function() {
            
            var a = this.concat();
            
            for( var i = 0; i < a.length; ++i ){
                
                for( var j = i + 1; j < a.length; ++j ){
                    
                    if( JSON.stringify( a[ i ] ) === JSON.stringify( a[ j ] ) )a.splice( j--, 1 );
                    
                }
                
            }

            return a;
        
        };

        // Private - definiamo la logica del debug
        function _lol( mex ){

			if( debug ){

				console.log( "[ " + THISCLASS.name + " ] " + mex );

			}

			return false;

		};

		// Private - utility per settare le opzioni
		function _reverseMerge( source, news ){

			source 	= source 	|| settings;
			news 	= news 		|| {};

			for( var key in source ) {

				if( source.hasOwnProperty( key ) && !( key in news )) {

			    	news[ key ] = source[ key ];

			    }

			}

			return news;

		};

		// Private - setto le opzioni
		function _setOptions( opt ){

			_lol( "Setto le nuove opzioni" );

			opt = opt || {};

			settings = _reverseMerge( settings, opt );

		};

		// Private - utility per clonare un oggetto
		function _clone( obj ){

			return $.extend( true, {}, obj );

		};
        
        // Private - Controlla se sono in una popup esterna
        function _iAmInNewPopUp(){
          
            return ( D.location.href.indexOf( settings.newPopUpTracker ) > -1 );
            
        };
        
        // Private - restituisco il parametro della nuova popup
        function _getNewPopUpParameters(){
            
            if( !_iAmInNewPopUp() )return null;
            
            var tmp = D.location.href.split( settings.newPopUpTracker );
            if( tmp.length != 2 )return null;
            
            var params = tmp[ 1 ].split( "/" );
            if( params.length != 3 )return null;
            
            return {
                
                url       : decodeURIComponent( params[ 0 ] ),
                windowId  : params[ 1 ],
                id        : params[ 2 ]
                
            };
            
        };
        
        // Private - informo che ho aperto la popup
        function _iAmOpen(){

            _lol( "Comunico al BG che sono in pertura" );
            chrome.runtime.sendMessage( { iam:settings.iam, dialog:"PopUpOpen" } );

        };

        // Private - chiedo la lista dei media
        function _getMediaGrabbed(){

            _lol( "Comunico al BG che voglio i media grabbed" );
            chrome.runtime.sendMessage( { iam:settings.iam, dialog:"GetMediaGrabbed" } );

        };

        // Private - carico il tema e visualizzo il body
        function _setThema( sets ){

            if( !sets )return false;

            _lol( "Setto il tema alla console" );
            if( sets.thema ){

                var thema = D.createElement( "link" );
                thema.setAttribute( "rel", "stylesheet" );
                thema.setAttribute( "type", "text/css" );
                thema.setAttribute( "href", sets.thema );
                thema.addEventListener( "load", function(){

                    _lol( "Visualizzo il body" );

                    $( "#gam-console" ).fadeIn();

                    _lol( "Richiedo i media catturati" );

                    _getMediaGrabbed();

                } );

                D.head.appendChild( thema );

            }else{

                _lol( "Non riesco a settare il tema" );

            }


        };
        
        // Private - visualizza la sezione default dei grabbed
        function _viewDefaultGrabbed(){
          
            // Predispongo tutte le div children
            $( "#grabed > div" ).hide();
            $( "#grabed > .default" ).fadeIn();
            
        };
        
        // Private - visualizzo la lista dei media catturati
        function _viewListGrabbed(){
          
            // Predispongo tutte le div children
            $( "#grabed > div" ).hide();
            $( "#grabed > .operative" ).show();
            
        };
        
        // Private - visualizzo la finestra di default dell'advanced
        function _viewDefaultAdvanced(){
          
            // Predispongo tutte le div children
            $( "#advanced > div" ).hide();
            $( "#advanced > .default" ).fadeIn();
            
        };
        
        // Private - visualizzo la finestra di scanning dell'advanced
        function _viewScanningAdvanced( reset ){
          
            if( reset ){
                
                // Resetto i finded
                finded = [];
                
                // Il layout
                _updateScanningInfo( "", 0, finded.length );
                
            }
            
            // Predispongo tutte le div children
            $( "#advanced > div" ).hide();
            $( "#advanced > .operative" ).show();
            
        };

        // Private - visualizzo la finestra lista dell'advanced
        function _viewListAdvanced(){
          
            // Predispongo tutte le div children
            $( "#advanced > div" ).hide();
            $( "#advanced > .list" ).show();
            
        };
        
        // Private - Seleziona la sezione grabbed e visualizza la schermata default
        function _showDefaultGrabbed(){
            
            // Cambio la schermata
            _viewDefaultGrabbed();
            
            // Predispongo il wrapper e visualizzo la lista
            $("[data-action='#grabed']").trigger( "click" );
            
        };
        
        // Private - Seleziona la sezione grabbed e visualizza la schermata delle liste
        function _showListGrabbed(){
            
            // Cambio la schermata
            _viewListGrabbed();
            
            // Predispongo il wrapper e visualizzo la lista
            $("[data-action='#grabed']").trigger( "click" );
            
        };
        
        // Private - Seleziona la sezione advanced e visualizza la default
        function _showDefaultAdvanced(){
            
            // Cambio la schermata
            _viewDefaultAdvanced();
            
            // Predispongo il wrapper e visualizzo la lista
            $("[data-action='#advanced']").trigger( "click" );
            
        };
        
        // Private - Seleziona la sezione advanced e visualizza l'elaborazione
        function _showScanningAdvanced(){
            
            // Cambio la schermata
            _viewScanningAdvanced();
            
            // Predispongo il wrapper e visualizzo la lista
            $("[data-action='#advanced']").trigger( "click" );
            
        };
        
        // Private - resetto la lista media e visualizzo il default
        function _resetListMediaToDefault(){
            
            // Resetto la lista
            $( "#grabed > .operative > .ui.animated.list" ).html( "" );
            
            // Mostro la default
            _viewDefaultGrabbed();
            
        };
        
        // Private - Seleziona la sezione advanced e visualizza la lista
        function _showListAdvanced(){
            
            // Cambio la schermata
            _viewListAdvanced();
            
            // Predispongo il wrapper e visualizzo la lista
            $("[data-action='#advanced']").trigger( "click" );
            
        };
        
        // Private - visualizziamo e settiamo la lista dei media grabbed
        function _macroListMediaGrabbed( sets ){

            if( ! sets )return false;

            _lol( "Abilito i comandi del menu" );
            $( ".content.main-l" ).fadeIn();

            var medialist = sets.medialist;

            if( medialist && medialist.length > 0 ){

                _lol( "Popolo la lista dei media" );
                _populateListMediaGrabbed( sets );
                _showListGrabbed();

            }else{

                _lol( "Non abbiamo nessun media, visualizzo la finestra di default" );
                _showDefaultGrabbed();

            }

        };
        
        // Private - riduce la lunghezza di una stringa
        function _minify( string, len ){
                    
            return ( !string || !len || string.length <= len ) ? string : string.substring( 0, len ) + " ...";
            
        };
        
        // Private - copia del testo sulla clipboard
        function _copyToClipBoard( text ){
            
            if( !text )return false;
            
            // Creo al volo un div dove inserire il testo da visualizzare
            var clipboard = D.createElement( "textarea" );
            
            // Lo rendiamo invisibile con la relativa  classe
            clipboard.setAttribute( "id", "gam-clipboard" );
            clipboard.setAttribute( "class", "clipboard" );
            
            D.body.appendChild( clipboard );
            
            // Ora seleziono il testo contenuto nella clipboard e lo copio
            $( "#gam-clipboard" ).html( text ).select();
  			document.execCommand( "Copy", false, null );
            
            // Rimuovo la clipboard
            $( "#gam-clipboard" ).remove();
            
            return true;
        };
        
        // Private - reperisco le informazioni basilari
        function _getBasicInfo(){
            
            var basicinfo = {};
            
            var manifest    = chrome.runtime.getManifest(),
                core        = _getCore(); 
            
            basicinfo.extension = manifest.version;
            basicinfo.core      = ( !core ) ? "0.0.0.0" : core.version;
            
            return basicinfo;
            
        };
        
        // Private - aggiunge semplicemente nuovi elementi
        function _populateListMediaGrabbed( sets ){
            
            if( ! sets )return false;
            
            var medialist = sets.medialist;

            if( medialist && medialist.length > 0 ){
                
                // Visualizzo la lista
                _viewListGrabbed();
                
                _lol( "Popolo la lista dei media" );
                for( var i = 0; i < medialist.length; i++ ){
                    
                    // Utilizzo il link come key, se esiste già proseguo
                    var idkey = MD5( medialist[ i ].url );
                    if( D.getElementById( idkey ) )continue;
                    
                    // Seleziono l'icona media o doc
                    var icon = "";
                    
                    switch( medialist[ i ].subtype ){
                            
                        case "video":
                            
                            icon = "video";
                            break;
                            
                        case "audio":
                            
                            icon = "volume down";
                            break;
                        
                        default :
                            
                            icon = "file";
                            
                    }
                    
                    // Preparo il record
                    var proto = settings.grabbedProto
                    .replace( /(\%ID\%)/gi, idkey )
                    .replace( /(\%I\%)/gi, icon )
                    .replace( /(\%E\%)/gi, medialist[ i ].ext )
                    .replace( /(\%S\%)/gi, medialist[ i ].size )
                    .replace( /(\%LL\%)/gi, medialist[ i ].url )
                    .replace( /(\%L\%)/gi, _minify( medialist[ i ].url, 55 ) );
                    
                    _lol( "Aggiungo un nuovo elemento alla console" );
                    
                    var basicinfo = _getBasicInfo();
                    
                    // Appendo il nuovo record
                    $( "#grabed > .operative > .ui.animated.list" ).append( proto );
                    
                    // Appendo l'evento copy link al nuovo record
                    $( "#" + idkey + " > .content > .icon.copy" ).click( function(){
                        
                        // Prelevo il link
                        var myurl    = $( this ).parent().find( "a" ).first().attr( "href" );
                        
                        // Copio nella clipboard, feedback true/false
                        var result = ( !_copyToClipBoard( myurl ) ) ? _translate( "mex13" ) : _translate( "mex14" );
                        
                        // Statistiche per vedere se viene utilizzata la feature
                        basicinfo.link = myurl;
                        
                        _gaq.push( [ "_trackEvent", "Link Copied", "Info", JSON.stringify( basicinfo ) ] );
                        
                        $( this ).parent().find( ".feedback.events" ).html( result ).fadeIn( 500, function(){
                            
                            $( this ).fadeOut( 500 );
                            
                        } );
                        
                    } );
                    
                    // Appendo l'evento per il download del link
                    $( "#" + idkey + " > .to-download" ).click( function(){
                        
                        // Prelevo il link
                        var myurl    = $( this ).parent().find( "a" ).first().attr( "href" );
                        
                        // Statistiche per vedere se viene utilizzata la feature
                        basicinfo.link = myurl;
                        
                        _gaq.push( [ "_trackEvent", "Link Downloaded", "Info", JSON.stringify( basicinfo ) ] );
                        
                        $( this ).parent().find( ".feedback.events" ).html( _translate( "mex15" ) ).fadeIn( 500, function(){
                            
                            var feedback = $( this );
                            
                            // Avvio il download
                            chrome.downloads.download({
                                
                                url 			: myurl,
                                conflictAction 	: "prompt",
                                saveAs 			: true
                            
                            }, function( downloadId ){
                                
                                var result = (typeof downloadId === "undefined") ? _translate( "mex16" ) : _translate( "mex17" );
                                                                   
                                feedback.html( result ).fadeOut( 1500 );
                                
                            }); 
                            
                        } );
                        
                    } );
                    
                }                

            }
            
        };
        
        // Private - aggiorno le indicazioni di scanning
        function _updateScanningInfo( current, percentage, finded ){
            
            current    = current    || "";
            percentage = percentage || 0;
            
            $( "#advanced > .operative > .current > .scanner" ).html( current );
            $( "#advanced > .operative > .progress > .bar" ).width( percentage + "%" );
            
            if( finded )$( "#advanced > .operative > .founded" ).html( finded );
            
        };
        
        // Private - restituisci una percentuale
        function _getPercentage( val, max ){
            
            if( !val || !max )return 100;

            var q = ( val / ( max / 100 ) );
            
            return ( 100 - q.toFixed( 0 ) );

        }
        
        // Private - scannerizzo i file caricati
        function _scannFileLoaded( test, exts, max, files, onEnd ){
                        
            files      = files      || [];
            max        = max        || files.length;
            onEnd      = onEnd      || function(){};
            
            // Non ho i riferimenti per il test, esco ...
            if( !test || !exts || files.length < 1 ){
                
                _updateScanningInfo( "Files Loaded", 100 );  
                
                return onEnd();
                
            }
            
            var percent = _getPercentage( files.length, max );
            
            // Iniziamo con informare
            _updateScanningInfo( _translate( "mex18" ), percent, finded.length );
            
            // Prelevo l'url e lo elimino dalla lista
            var exame = files[ 0 ];
            files.splice( 0, 1 );
            
            // Se non dovessi averlo vado avanti
            if( !exame.url )return _scannFileLoaded( test, exts, max, files, onEnd );
            
            // Invio la richiesta    
            return $.ajax( {

                type     : "GET",
                url      : exame.url,
                dataType : "html",
                cache    : true,
                success  : function( data ){				
                    
                    if( !data )return _scannFileLoaded( test, exts, max, files, onEnd );
                    
                    data = data.trim();
                    
                    var firstTest  = new RegExp( test, "gi" ),
                        secondTest = new RegExp( "(" + exts + ")", "gi" );
                    
                    _lol( "Test 1 : " + test );
                    _lol( "Test 2 : " + exts );
                    
                    if( data.match( firstTest ) || data.match( secondTest ) ){
                        
                        // Aggiungo alcune informazioni da visualizzare il futuro
                        exame.data       = data;
                        exame.findedtype = "file";
                        
                        // Qui dobbiamo incrementare i risultati
                        finded.push( exame );                    
                    }

                    return _scannFileLoaded( test, exts, max, files, onEnd );

                },
                error    : function() {

                    return _scannFileLoaded( test, exts, max, files, onEnd );

                } 

            });
            
        };
        
        // Private - aggiorno la lista dei risultati dello scanner
        function _populateListAdvanced(){
            
            if( finded.length < 1 )return false;
            
            // Resetto la lista
            $( "#advanced > .list > .list" ).html( "" );
            
            for( var i = 0; i < finded.length; i++ ){
                
                // Preparo un item
                var proto = settings.findedProto
                    .replace( /(\%I\%)/gi, i )
                    .replace( /(\%H\%)/gi, finded[ i ].ext )
                    .replace( /(\%S\%)/gi, _minify( finded[ i ].data, 55 ) );
                
                // Lo aggiungo alla lista
                $( "#advanced > .list > .list" ).append( proto );
                
            }
            
            // Aggiungo gli eventi per tutti gli item
            $( "#advanced > .list > .list > .item > .browser.icon" ).click( function(){
                
                var data = finded[ parseInt( $( this ).attr( "data-index" ) ) ].data,
                    url  = finded[ parseInt( $( this ).attr( "data-index" ) ) ].url || "";
                
                chrome.windows.create( {

                    url     : settings.file.details + settings.newDTTracker + encodeURIComponent( data ) + settings.newDTTracker + url, 
                    width   : 800, 
                    height  : 700,
                    focused : true,
                    type    : "panel"

                } );
                
            } );
            
        };
        
        // Private - popola la lista dei advanced se finded
        function _finalizeAdvanced( sources ){
            
            // Sommo i 2 array con valori univoci perchè più popup potrebbero inviare messaggi duplicati
            if( sources && sources.length > 0 )finded = finded.concat( sources ).uniqueObj();
            
            _lol( "Abbiamo : " + finded.length );
            
            if( finded.length > 0 ){

                // Preparo la lista dei risultati
                _populateListAdvanced();                                

                // Visualizzo la lista
                _viewListAdvanced();

            }else{

                _viewDefaultAdvanced();

            }
            
        };
        
        // Private - controllo che il core sia stato inizializzato
        function _isCoreInitailized(){
            
            return !( !localStorage[ settings.storage.core ] || typeof localStorage[ settings.storage.core ] === "undefined" );
            
        }
        
        // Private - prelevo il core e restituisco un oggetto JSON
        function _getCore(){
            
            return ( _isCoreInitailized() ) ? JSON.parse( localStorage[ settings.storage.core ] ) : null;
            
        };
        
        // Private - preleva la traduzione corrispondente
        function _translate( key ){
          
            return ( key && typeof key == "string" )? chrome.i18n.getMessage( key ) : "";
            
        };
        
        // Private - traduci alcuni elementi base nella pagina della console
        function _translateConsole(){
            
            $( "#grabed > .default > .title" ).html( _translate( "mex02" ) );
            $( "#grabed > .default > .dectails" ).html( _translate( "mex03" ) );
            $( "#advanced > .default > .title" ).html( _translate( "mex04" ) );
            $( "#advanced > .default > .dectails" ).html( _translate( "mex05" ) );
            $( "#default > .default > .loading" ).html( _translate( "mex06" ) );
            $( "#advanced > .default > .scanner > .content.check" ).html( _translate( "mex07" ) );
            $( "#advanced > .operative > .title" ).html( _translate( "mex08" ) );
            $( "#advanced > .operative > .current > .dectails" ).html( _translate( "mex09" ) );
            $( "#bookmarklets > .default > .title" ).html( _translate( "mex10" ) );
            $( "#bookmarklets > .default > .dectails" ).html( _translate( "mex11" ) );
            $( ".bookmarklets" ).attr( "title", _translate( "mex12" ) );
            
        };
        
		// Public Shared - passo in modalità debug, registro gli eventi
		obj.DebugMode = function( value ){

			value = value || false ;

			debug = value;

			_lol( "DebugMode : " + value );

		};

		// Public Shared - appendo gli eventi a tutti gli oggetti caricati, eseguibile una sola volta
        obj.Initialize = function(){

            // Se ho già inizializzato non vado oltre
            if( initialized )return false;

            _lol( "Inizializzo la console" );
            
            // Traduco i contenuti della console
            _translateConsole();
            
            var backEvents = function(){               
                
                // Appendiamo gli eventi per il menu
                $( ".ui.menu.main-l > .item" ).click( function(){

                    // Rimuovo tutti i stati attivi
                    $( ".ui.menu.main-l > .item" ).removeClass( "active" );

                    // Aggiungo lo stato attivo per il selezionato
                    $( this ).addClass( "active" );

                    // Nascondo tutti i wrapper
                    $( ".content.main-r > .wrapper > div" ).hide();

                    // Visuaizzo il wrapper selezionato
                    var action = $( this ).attr( "data-action" );

                    if( action.toLowerCase() == "#settings" ){

                        chrome.windows.create( {

                            url     : settings.file.options, 
                            width   : 800, 
                            height  : 600,
                            focused : true,
                            type    : "panel"

                        } );

                        // Torno alla finestra advanced
                        if( $( "#grabed > .operative > .ui.animated.list" ).html().trim().length > 0 ){
                            
                            _showListGrabbed();
                            
                        }else{
                            
                            _showDefaultGrabbed();
                            
                        }

                    }else{

                        $( $( this ).attr( "data-action" ) ).fadeIn();

                    }

                } );
                
                // Appendiamo l'evento per lo scanner
                $( "#advanced > .list > .mini-menu > .right.icon" ).click( function(){
                    
                    // Visualizzo la schermata default
                    _viewDefaultAdvanced();
                    
                } );
                
                $( ".advanced.scanner" ).click( function(){
                    
                    // Visualizziamo la schermata di scanning
                    _viewScanningAdvanced( true );
                    
                    // Scannerizziamo i file caricati, voglio coinvolgere il BG anche per le altre popups
                    _lol( "Comunico al BG di voler scannerizzare" );
                    chrome.runtime.sendMessage( { iam:settings.iam, dialog:"PopUp-Scann-Files-Loaded" } );
                    
                    
                } );
                
                // Inizializzo gli elementi dei forms
                $( ".ui.checkbox" ).checkbox();
                $( ".ui.selection.dropdown" ).dropdown();
                
                // Eventi per i bookmarklets
                $( ".bookmarklets" ).click( function(){
                    
                    alert( _translate( "mex12" ) );
                    return false;
                    
                } );
                
                // Informiamo il bg che siamo aperti
                _iAmOpen();
                
            };
            
            // Adatto il contenuto alla finestra e aggiorno il titolo, se sono in esterna
            if( _iAmInNewPopUp() ){
                
                var tab   = _getNewPopUpParameters();
                
                // Se non ho i parametri esco
                if( !tab ){
                    
                    alert( _translate( "mex19" ) );
                    window.close();
                    
                }else{
                    
                    // Aggiorno il mio id (iam)
                    settings.iam   = MD5( tab.id );                    
                    settings.tabId = parseInt( tab.id );
                    
                    // Aggiorno il titolo della finestra                
                    D.title = "Grab Any Media | " + tab.url;//chrome.runtime.getManifest().version;
                    
                    // Non permetto il resize
                    $( window ).resize( function(){

                        _lol( "Aggiusto il contenuto nella finestra" );  
                        window.resizeTo( $( "#gam-console" ).width() + 20, $( "#gam-console" ).height() + 40 );

                    } );  
                    
                    // Finalizzo
                    backEvents();
                    
                }               
                
            }else{
                
                // Se non sono in una nuova popup devo prelevare la tab aperta, asincrono
                chrome.tabs.query( {

                            windowId: chrome.windows.WINDOW_ID_CURRENT,
                            active: true

                        },function ( tabs ){

                            if( tabs.length > 0 ){
                                
                                settings.iam   = MD5( tabs[0].id.toString() );
                                settings.tabId = tabs[0].id;
                            
                            }
                            
                            // Comunico il mio id come main, mutevole, non utilizzato finora
                            chrome.runtime.sendMessage( { iam:settings.iam, dialog:"SetMainPopUpExtension" } );
                            
                            // Finalizzo
                            backEvents();                           
                            
                        }

                );
                
            }            

            initialized = true;

        };

        // Public Shared - attivo i listener ( comunicazione con il background, ed altro ... )
        obj.Listen = function(){
                        
            _lol( "Mi metto in ascolto con il BG" );
            chrome.runtime.onMessage.addListener( function( request, sender, sendResponse ){
                
                // Il messaggio non è indirizzato a me
                if( request.for != "*" && request.for != settings.iam )return false;
                
                switch( request.dialog ){
                    
                    case "Update-Score-Advanced":
                        
                        // Devo aggiornare lo score dei test
                        var percent = _getPercentage( request.sets.current, request.sets.max );
            
                        // Iniziamo con informare
                        _updateScanningInfo( _translate( "mex20" ), percent, finded.length + request.sets.founded );
                        
                        break;
                        
                    case "End-Scanning-Advanced":
                        
                        _lol( "Source ha trovato : " + request.finded.length );
                        _finalizeAdvanced( request.finded );
                        
                        break;
                        
                    case "PopUpScannFilesLoaded":           
                        
                        // devo almeno passare alla schermata di scanning
                        _viewScanningAdvanced( true );              
                        
                        // Scannerizzo i file caricati
                        _lol( "Scannerizzo i files" );
                        _scannFileLoaded( request.sets.test, request.sets.exts, null, request.sets.files, function(){
                                            
                            // Test per inniettare il source                               
                            if( settings.tabId > -1 ){
                                
                                // Predispongo il loader
                                _updateScanningInfo( _translate( "mex20" ), 0, null );
                                
                                // Innietto prima qualche informazione utile
                                chrome.tabs.executeScript( settings.tabId, { 
                                    
                                    code : "window.GAMinfo = { popup : \"" + settings.iam + "\", core : " + JSON.stringify( _getCore() ) +  " };" 
                                
                                }, function(){
                                    
                                    // Poi innietto lo script generale
                                    chrome.tabs.executeScript( settings.tabId, { file:settings.inject.advanced } );
                                    
                                } );                                
                            
                            }else{
                                
                                _finalizeAdvanced();
                                
                            }
                                
                        } );
                        
                        break;
                        
                    case "PopUp-Refresh-All":
                        
                        // Aggiorniamo la popup
                        location.reload();
                        
                        break;
                        
                    case "PopUpClosed":
                        
                        window.close();
                        
                        break;
                        
                    case "PopUpResetToDefaultMediagrabbed":
                        
                        // Devo resettare la lista media e visualizzare il default
                        _resetListMediaToDefault();
                        
                        break;
                        
                    case "PopUpSetThema":

                        _lol( "Ricevo il comando di settare il tema" );
                        _setThema( request.sets );

                        break;

                    case "PopUpSetListMediaGrabbed":

                        _lol( "Ricevo il comando di aggiornare la lista dei media grabbed" );
                        _macroListMediaGrabbed( request.sets );

                        break;
                    
                    case "PopUpPopulateMediagrabbed":
                        
                        _lol( "Aggiorno la lista dei grabbed" );
                        _populateListMediaGrabbed( request.sets );
                        
                        break;

                
                }

            } );
                        
        };


		return obj;

	};// Fine ClassGAMconsole

	window.GAMconsole = ClassGAMconsole();

})( document, window.jQuery );




// Aspetto che tutti gli elementi siano caricati
window.onload = ( function( CONSOLE ){

    // Attivo il debug
    CONSOLE.DebugMode( false );

    // Inizializzo i componenti
    CONSOLE.Initialize();

    // Metto in ascolto con il background
    CONSOLE.Listen();

} )( window.GAMconsole );








