/*


	SHARED 		: comune a tutte le istanze
	PRIVATE 	: non visibile
	PUBLIC 		: visibile dopo instanza.
	
	this. 		: richiamabile senza instanziare
	prototype. 	: richiamabile solo se instanziato

*/

(function( D, $ ){
	
	// Private Shared - dati identificativi di classe
	var THISCLASS = {

		version : "5.0.0.0",
		name 	: "gam-options.js",

	};

	if( !D ) throw new Error( "[ " + THISCLASS.name + " ] require document loaded !" );
	if( !$ ) throw new Error( "[ " + THISCLASS.name + " ] require jQuery loaded !" );
    
    // Shared - variabile di controllo per il debug, per tutte le istanze
	var debug       = false,
        initialized = false,    // Per eventi unici
        mainsite    = "http://grabanymedia.altervista.org/";

	// Private - definisco la classe
	function ClassGAMopt(){
        
        // L'oggetto con le funzioni da restituire
        var obj = {};
        
		// Private - settaggi
		var settings = {
            
            iam     : new Date().getTime(), // Timestamp come id univoco
            storage : {
                
                settings    : "settings",
                core        : "core"
                
            },
            shares 		: {
                
                facebook 	: "https://www.facebook.com/GrabAnyMedia",
                twitter 	: "https://twitter.com/GrabAnyMedia",
                google 		: "https://plus.google.com/111904112063524443262",
                pinterest	: "http://www.pinterest.com/leonardociaccio/grab-any-media/",
                youtube 	: "http://www.youtube.com/playlist?list=PLISSxErRXU1OdHPJfX3htWDLWs1D4yUsx"
                
            }
            
        };
        
        // Private - definiamo la logica del debug
        function _lol( mex ){

			if( debug ){

				console.log( "[ " + THISCLASS.name + " ] " + mex );

			}
				
			return false;

		};
		
		// Private - utility per settare le opzioni
		function _reverseMerge( source, news ){

			source 	= source 	|| settings;
			news 	= news 		|| {};	

			for( var key in source ) {
			    
				if( source.hasOwnProperty( key ) && !( key in news )) {
			     
			    	news[ key ] = source[ key ];
			    
			    }
			  
			}
			
			return news;
		
		};

		// Private - setto le opzioni
		function _setOptions( opt ){

			_lol( "Setto le nuove opzioni" );

			opt = opt || {};

			settings = _reverseMerge( settings, opt );

		};

		// Private - utility per clonare un oggetto
		function _clone( obj ){

			return $.extend( true, {}, obj );

		};
        
        // Private - controllo che i settaggi siano inizializzati
        function _isSettingsInitailized(){
            
            return !( !localStorage[ settings.storage.settings ] || typeof localStorage[ settings.storage.settings ] === "undefined" );
            
        }
        
        // Private - prelevo i settaggi e restituisco un oggetto JSON
        function _getSettings(){
            
            return ( _isSettingsInitailized() ) ? JSON.parse( localStorage[ settings.storage.settings ] ) : null;
            
        };
        
        // Private - installo i settaggi passati
        function _setSettings( sets ){
                
            // Controllo approfondito
            if( !sets || typeof sets != "object" || !sets.marker || sets.marker != "settings" )return false;
            
            _lol( "Installo i settaggi" );
            localStorage[ settings.storage.settings ] = JSON.stringify( sets );
            
            // Comunico ad eventuali popup di chiudersi
            chrome.runtime.sendMessage( { iam:settings.iam, dialog:"Close-Window" } );
            
            // Aggiorno il context menu
            chrome.runtime.sendMessage( { iam:settings.iam, dialog:"UpdateContextMenu" } );
            
            return true;
            
        };
        
        // Private - controllo che il core sia stato inizializzato
        function _isCoreInitailized(){
            
            return !( !localStorage[ settings.storage.core ] || typeof localStorage[ settings.storage.core ] === "undefined" );
            
        }
        
        // Private - prelevo il core e restituisco un oggetto JSON
        function _getCore(){
            
            return ( _isCoreInitailized() ) ? JSON.parse( localStorage[ settings.storage.core ] ) : null;
            
        };
        
        // Private - visualizza uno spot
        function _showSpot( selector, text ){
            
            if( !selector || !text )return false;
            
            $( selector ).html( text ).fadeIn( 500, function(){
             
                $( this ).fadeOut( 1000 );
                
            } );
            
        }
        
        // Private - preleva la traduzione corrispondente
        function _translate( key ){
          
            return ( key && typeof key == "string" )? chrome.i18n.getMessage( key ) : "";
            
        };
        
        // Private - traduci alcuni elementi base nella pagina della console
        function _translateOptions(){
            
            document.title = _translate( "mex36" );
            $( ".header > .content > .title" ).html( _translate( "mex37" ) );
            $( ".header > .content > .sub.header" ).html( _translate( "mex38" ) );
            $( ".header.main > .title" ).html( _translate( "mex39" ) );
            $( ".header.styles > .title" ).html( _translate( "mex40" ) );            
            $( ".t-d-grab" ).html( _translate( "mex56" ) );           
            $( ".t-red" ).html( _translate( "mex41" ) );
            $( ".t-blue" ).html( _translate( "mex42" ) );
            $( ".t-black" ).html( _translate( "mex43" ) );
            $( ".header.media-size > .title" ).html( _translate( "mex44" ) );
            $( ".max-size > .default.text" ).html( _translate( "mex45" ) );
            $( ".max-size > .menu > .item.all" ).html( _translate( "mex46" ) );
            $( ".header.files-type > .title" ).html( _translate( "mex47" ) );
            $( ".bdg.version" ).html( _translate( "mex48" ) );
            $( ".bdg.core" ).html( _translate( "mex49" ) );
            $( ".bdg.framework" ).html( _translate( "mex50" ) );
            
        };
        
        
        // Private - nascondo/visualizzo i parametri per i don't grab
        function _enableDisableSettings( enable ){
                        
            if( !enable ){
                
                $( ".for.files-size" ).fadeOut( 500 );
                $( ".for.files-type" ).fadeOut( 500 );
                
            }else{
                
                $( ".for.files-size" ).fadeIn( 500 );
                $( ".for.files-type" ).fadeIn( 500 );
                
            }           
            
        };
        
		// Public Shared - passo in modalità debug, registro gli eventi
		obj.DebugMode = function( value ){

			value = value || false ;

			debug = value;

			_lol( "DebugMode : " + value );

		};

		// Public Shared - appendo gli eventi a tutti gli oggetti caricati, eseguibile una sola volta
        obj.Initialize = function(){
            
            // Se ho già inizializzato non vado oltre
            if( initialized )return false;
            
            // Traduco la pagina
            _translateOptions();
            
            // Inizializzo gli elementi dei forms edella pagina
            $( ".ui.checkbox" ).checkbox();
            $( ".ui.selection.dropdown" ).dropdown();
            $( ".ui.sidebar" ).sidebar( {
                
                overlay: true
            
            } ).sidebar( "toggle" );
            $( ".popups" ).popup();
            
            // Per prima cosa prelevo i settaggi e setto in posizione i valori
            var sets = _getSettings();
            
            // Se non ho i settaggi è inutile continuare
            if( !sets ){
                
                alert( _translate( "mex51" ) );
                window.close();
                return false;
                
            }
            
            // Ho i settaggi setto i valori
            var state = ( !sets.checkDG ) ? "disable" : "enable";
            $( ".checkbox.dont-grab" ).checkbox( state );
            
            // Abilito o disabilito le altre opzioni
            _enableDisableSettings( !sets.checkDG );
            
            // Setto il tema salvato
            var mythema = sets.thema;
            if( mythema )$( "input:radio[name=thema]" ).filter( "[value=" + mythema + "]" ).prop( "checked", true );
            
            // Settiamo le dimensioni massime dei file
            var maxsize = sets.maxFileSize;
            $( ".dropdown.max-size" ).dropdown( "set selected", maxsize );
            
            // Setto il file JS
            state = ( !sets.checkJS ) ? "disable" : "enable";
            $( ".checkbox.files-js" ).checkbox( state );
            
            // Setto il file JSON
            state = ( !sets.checkJSON ) ? "disable" : "enable";
            $( ".checkbox.files-json" ).checkbox( state );
            
            // Setto il file XML
            state = ( !sets.checkXML ) ? "disable" : "enable";
            $( ".checkbox.files-xml" ).checkbox( state );
            
            // Setto il file SWF
            state = ( !sets.checkSWF ) ? "disable" : "enable";
            $( ".checkbox.files-swf" ).checkbox( state );
            
            // Prelevo la versione della estensione
            var manifest 	= chrome.runtime.getManifest();
            $( ".detail.extension" ).html( manifest.version );
            
            // Prelevo la versione del core
            var core    = _getCore(),
                version = ( !core ) ? _translate( "mex52" ) : core.version;
            $( ".detail.core" ).html( version );
                        
            // Eventi per il don't grab
            $( ".checkbox.dont-grab" ).checkbox( "setting", "onEnable", function(){
                
                sets.checkDG = true;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                // Comunico al BG di aggiornare l'icona
                chrome.runtime.sendMessage( { dialog:"IconUpdate" } );
                
                // Abilito gli altri comandi
                _enableDisableSettings();     
                
                _showSpot( ".ui.header.main > .events", response );
                
            } ).checkbox( "setting", "onDisable", function(){
                
                sets.checkDG = false;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                // Comunico al BG di aggiornare l'icona
                chrome.runtime.sendMessage( { dialog:"IconUpdate" } );
                
                // Disabilito gli altri comandi
                _enableDisableSettings( true );
                
                _showSpot( ".ui.header.main > .events", response );
                
            } );
            
            // Eventi per lo style
            $( ".styles > .field > .ui.radio.checkbox" ).checkbox( "setting", "onChange", function( value ){
                
                // Aggiorno il thema nei settaggi
                var mythema = $( this ).val();
                if( !mythema ){
                    
                    _showSpot( ".ui.header.styles > .events", _translate( "mex55" ) );
                    
                }else{
                    
                    sets.thema = mythema;
                    var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                    
                    // Refresh delle popups
                    chrome.runtime.sendMessage( { for:"*", dialog:"PopUp-Refresh-All" } );
                    
                    _showSpot( ".ui.header.styles > .events", response );
                    
                }
                
            } );
            
            // Eventi per il filesize
            $( ".dropdown.max-size" ).dropdown( {
                
                onChange : function( value, text ){
                    
                    if( sets.maxFileSize == value )return false;
                    
                    sets.maxFileSize = value;
                    var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );

                    _showSpot( ".ui.header.media-size > .events", response );    
                    
                }
                
            } );
            
            // Eventi per JS
            $( ".checkbox.files-js" ).checkbox( "setting", "onEnable", function(){
                
                sets.checkJS = true;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                _showSpot( ".ui.header.files-type > .events", response );
                
            } ).checkbox( "setting", "onDisable", function(){
                
                sets.checkJS = false;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                _showSpot( ".ui.header.files-type > .events", response );
                
            } );
            
            // Eventi per JSON
            $( ".checkbox.files-json" ).checkbox( "setting", "onEnable", function(){
                
                sets.checkJSON = true;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                _showSpot( ".ui.header.files-type > .events", response );
                
            } ).checkbox( "setting", "onDisable", function(){
                
                sets.checkJSON = false;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                _showSpot( ".ui.header.files-type > .events", response );
                
            } );
            
            // Eventi per XML
            $( ".checkbox.files-xml" ).checkbox( "setting", "onEnable", function(){
                
                sets.checkXML = true;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                _showSpot( ".ui.header.files-type > .events", response );
                
            } ).checkbox( "setting", "onDisable", function(){
                
                sets.checkXML = false;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                _showSpot( ".ui.header.files-type > .events", response );
                
            } );
            
            // Eventi per SWF
            $( ".checkbox.files-swf" ).checkbox( "setting", "onEnable", function(){
                
                sets.checkSWF = true;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                _showSpot( ".ui.header.files-type > .events", response );
                
            } ).checkbox( "setting", "onDisable", function(){
                
                sets.checkSWF = false;
                var response = ( !_setSettings( sets ) ) ? _translate( "mex53" ) : _translate( "mex54" );
                
                _showSpot( ".ui.header.files-type > .events", response );
                
            } );
            
            // Eventi per le share icons
            $( ".share > .icon.facebook" ).click( function(){
                
                _lol( "Condividi su facebook : " + settings.shares.facebook );
                window.open( settings.shares.facebook, "_blank" );
                
            } );
            
            $( ".share > .icon.google.plus" ).click( function(){
                
                _lol( "Condividi su google : " + settings.shares.google );
                window.open( settings.shares.google, "_blank" );
                
            } );
            
            $( ".share > .icon.twitter" ).click( function(){
                
                _lol( "Condividi su twitter : " + settings.shares.twitter );
                window.open( settings.shares.twitter, "_blank" );
                
            } );
            
            $( ".share > .icon.pinterest" ).click( function(){
                
                _lol( "Condividi su pinterest : " + settings.shares.pinterest );
                window.open( settings.shares.pinterest, "_blank" );
                
            } );
            
            $( ".share > .icon.youtube.play" ).click( function(){
                
                _lol( "Condividi su youtube : " + settings.shares.youtube );
                window.open( settings.shares.youtube, "_blank" );
                
            } );
            
            _lol( "Inizializzo le OPT" );
            
            initialized = true;
            
        };
        
        // Public Shared - attivo i listener ( comunicazione con il background, ed altro ... )
        obj.Listen = function(){
                        
            _lol( "Mi metto in ascolto con le popup" );            
            chrome.runtime.onMessage.addListener( function( request, sender, sendResponse ){
		
                switch( request.dialog ){
                    
                    case "Close-Window":
                        
                        if( request.iam != settings.iam ){
                            
                            // Chiudo questa finestra
                            window.close();
                        
                        }
                        
                        break;

                }

            } );
            
        };
        
		return obj;

	};// Fine ClassGAMopt

	window.GMAopt = ClassGAMopt();

})( document, window.jQuery );




// Aspetto che tutti gli elementi siano caricati
window.onload = ( function( OPT ){
    
    // Attivo il debug
    OPT.DebugMode( false );
    
    // Inizializzo i componenti
    OPT.Initialize();
    
    // Mi metto in ascolto
    OPT.Listen();
        
} )( window.GMAopt );











